#!/bin/bash

### COPYRIGHT NOTICE ###
# Copyright (C) 2006-2008 TOLIS Group, Inc.  All rights reserved.

### DISCLAIMER ###
# TOLIS provides this script and assumes no responsibility
# for the content or operation in any particular environment.
# Further, TOLIS assumes no liability for potential harm or
# damage, software or hardware, that may result from the use of
# this script. The script are provided as-is only and may be
# modified for your use. VERIFY THE USEFULNESS of the script
# before you use it. The downloading and use of this script
# indicates that you have read and agree to these terms.

### ABOUT THIS SCRIPT ###
# This script will run through 0, 32k, 64k, 128k, 256k, and
# 512k block sizes as well as different BRU BUFSIZE settings
# to find the best speed settings for your installation.



#################################################
# Do not change anything below unless you know  #
# what you are doing!                           #
#################################################


# Sort and sanitize the command line arguments.
# Arg count should only be 3 (script, device, path).
if [ $# -ne 2 ]
then
    if [ $# -gt 0 ]
    then
    	printf "\n\aInvalid argument: \"$@\"\n"
    fi
    printf "\n  Usage: speed_test.sh <device> <test_path>\n"
    printf "  example: speed_test.sh ntape0 /Users/admin/Documents/\n"
    printf "\n"
    exit 255
fi


### VARIABLES ###
# DO NOT CHANGE THESE!
LOG="speedlog.txt"
OLDLOG="speedlog_old.txt"
DEVICE=$1
BACKUP_PATH=$2
ROOT_UID=0
E_XCD=66
E_NOTROOT=67
EXIT_ERR=`echo $?`

clear

# Check if running as Root.
if [ "$UID" -ne "$ROOT_UID" ]
then
  printf "\tMust be root to run this script.\n"
  exit $E_NOTROOT
fi

printf "TOLIS BRU SPEED TEST SCRIPT\n\n"

# Clear existing speedlog.txt file?
if [ -e $LOG ]
then
	printf "Rename existing $LOG file? [N/y] "
	read ANS
	if [ "${ANS}" == "y" -o "${ANS}" == "Y" ]
		then
			mv $LOG $OLDLOG
			touch 
			printf "Renamed to '$OLDLOG'.\n"
			printf "==============================\n" >> $LOG
			printf "    NEW SPEED TEST RESULTS\n" >> $LOG
			printf "==============================\n" >> $LOG
			wait
	else
		printf "Appending to log file $LOG.\n"
		printf "\n\n\n\n" >> $LOG
		printf "==============================\n" >> $LOG
		printf "    NEW SPEED TEST RESULTS\n" >> $LOG
		printf "==============================\n" >> $LOG
		printf "\n" >> $LOG
		wait
		sleep 2
	fi
fi

clear

# Show disclaimer just to be on the safe side.
printf "TOLIS BRU SPEED TEST SCRIPT\n\n"
printf "### DISCLAIMER ### \n \
 TOLIS provides this script and assumes no responsibility \n \
 for the content or operation in any particular environment. \n \
 Further, TOLIS assumes no liability for potential harm or \n \
 damage, software or hardware, that may result from the use of \n \
 this script. The script are provided as-is only and may be \n \
 modified for your use. VERIFY THE USEFULNESS of the script \n \
 before you use it. The downloading and use of this script \n \
 indicates that you have read and agree to these terms.\n\n"
printf "Do you agree?\n"
printf "If yes, please type \"yes\": "
read ANS
if [ "${ANS}" != "yes" ]
then
	printf "Entered value is not \"yes\".\n"
	printf "Exiting...\n"
	exit 0
else
	printf "\n" >> $LOG
	printf "User AGREED to DISCLAIMER of speedtest.sh script.\n" >> $LOG
	printf "=================================================" >> $LOG
	printf "\n" >> $LOG
fi

clear

# Display information about this script that may be of use.
printf "TOLIS BRU SPEED TEST SCRIPT\n\n"
printf "### INFORMATION ###\n \
This script is best suited for backing up a directory that has at\n \
least 5GB of data to use as test backup data.  Some directories\n \
that are good for testing are directories with large files such\n \
as ISO's, movies, large pictures, songs, etc.\n\n \
Remote or network volumes should not be used for tape drive performance\n \
testing due to network I/O involvement.  Remote or network volumes\n \
may slow down these speed tests, resulting in inaccruate performance\n \
numbers.\n\n \
However, if you are running these test to find out what speeds work best\n \
for your real-world backups, you should choose a directory or path that\n \
that will be backed up on a regular basis with BRU.\n\n\
The path that has been selected for backup is:\n\n\
\t$BACKUP_PATH\n\n\
Would you like to continue with the desired path? [N/y] "
read ANS
if [ "${ANS}" != "y" -a "${ANS}" != "Y" ]
then
	printf "\nPlease rerun the script and specify the path as desired.\n"
	printf "Exiting...\n\n"
	exit 0
fi

clear

# Select if this is OS X or Linux.
printf "TOLIS BRU SPEED TEST SCRIPT\n\n"
printf "Please specify system type:\n"
printf "  1. Mac OS X -- tapectl command\n"
printf "  2. Linux -- mt command\n"
printf "Select an option: "
read ANS

if [ "${ANS}" == "1" ]
then
	CMD=tapectl
	DBUFFER="no"
	BRUCMD="bru -cvvvBf $DEVICE -b $X $BACKUP_PATH"
	printf "\nMac OS X chosen as OS. 'tapectl' command will be used for tape control.\n"
elif [ "${ANS}" == "2" ]
then
	printf "Linux chosen as OS.  'mt' command will be used for tape control.\n"
	printf "Would you like to test using Double Buffering (-D)?  [N/y] "
	read ANS
	if [ "${ANS}" != "y" -a "${ANS}" != "Y" ]
	then
		printf "Double Buffering will NOT be used."
		CMD=mt
		DBUFFER="no"
	else
		printf "Double Buffering WILL be used."
		CMD=mt
		DBUFFER="yes"
	fi
else
	printf "\nInvalid option.\n"
	printf "Exiting Speed Test.\n\n"
	exit 0
fi

wait

# Exit now if desired.
printf "\nIf you wish to exit, press CONTROL-C now....\n"
sleep 5
printf "Moving on!"
sleep 1

clear

# Last change for overwrite protect and exit.
printf "Please make sure there is a tape that can be OVERWRITTEN in $DEVICE.\n\n"
printf "This is the last chance to stop this script and prevent any overwrite\n"
printf "of data that may be on the tape used for speed tests.\n\n"
printf "Continue? [N/y]: "
read ANS

if [ "${ANS}" != "Y" -a "${ANS}" != "y" ]
then
	printf "Exiting script...\n"
	sleep 2
	exit 0
else
	clear
	printf "Running Speed Tests...\n"
	printf "This will take a while!\n"
fi

# Specify the rewind command based on the OS chosen.
REWIND="$CMD -v -f $DEVICE rewind"
printf "Backup path is: $BACKUP_PATH\n"
printf "Backup path is: $BACKUP_PATH\n" >> $LOG
printf "Using drive: $DEVICE\n"
printf "Using drive: $DEVICE\n" >> $LOG
printf -- "------------------------------\n"
printf -- "------------------------------\n" >> $LOG

# Actual Speed Test Process.
for BLK in 0 32768 65536 131072 262144 524288
do
 	$REWIND
 	# Set the tape drive to the correct fixed block mode.
 	$CMD -f $DEVICE setblk $BLK
 	printf "\n" >> $LOG
 	printf "NEW TAPE BLOCK TEST\n"
 	printf "NEW TAPE BLOCK TEST\n" >> $LOG
 	printf "==================================\n"
 	printf "==================================\n" >> $LOG
 	printf "\nTape block size set to $BLK\n"
 	printf "Tape block size set to $BLK\n" >> $LOG
 	if [ $BLK -lt "262143" ]
 	then
    BUFS="128k 256k 512k 1024k 2048k"
 	elif [ $BLK = "262144" ]
 	then
    BUFS="256k 512k 1024k 2048k"
 	else
    BUFS="512k 1024k 2048k"
 	fi

 	for X in $BUFS
  do
		if [ "${DBUFFER}" == "no" ]
		then
			BRUCMD="bru -cvvvBf ${DEVICE} -b ${X} ${BACKUP_PATH}"
		elif [ "${DBUFFER}" == "yes" ]
		then
			BRUCMD="bru -cvvvBf ${DEVICE} -D -b ${X} ${BACKUP_PATH}"
			printf "Double-Buffering Enabled.\n" >> $LOG
			printf "Double-Buffering Enabled.\n"
		else
			printf "Double Buffer Variable Error.\n"
			printf "Exiting Speed Test...\n"
			exit 1
		fi
    printf "Performing $X BRU buffer test at $BLK tape block size.\n"
    printf "Performing $X BRU buffer test at $BLK tape block size.\n" >> $LOG
    printf -- "--------------------------------\n"
    $REWIND
    printf -- "--------------------------------\n"
    printf "Running $X BRU buffer test 1...\n"
    printf "Running $X BRU buffer test 1...\n" >> $LOG
    $BRUCMD | tail -1 >> $LOG
    printf "$X BRU buffer test 1 complete.\n" >> $LOG
    printf "$X BRU buffer test 1 complete.\n"
    printf -- "--------------------------------\n"
    $REWIND
    printf -- "--------------------------------\n"
    printf "Running $X BRU buffer test 2...\n"
    printf "Running $X BRU buffer test 2...\n" >> $LOG
    $BRUCMD | tail -1 >> $LOG
    printf "$X BRU buffer test 2 complete.\n" >> $LOG
    printf "$X BRU buffer test 2 complete.\n"
    printf -- "--------------------------------\n"
    $REWIND
    printf -- "--------------------------------\n"
    printf "Running $X BRU buffer test 3...\n"
    printf "Running $X BRU buffer test 3...\n" >> $LOG
    $BRUCMD | tail -1 >> $LOG
    printf "$X BRU buffer test 3 complete.\n" >> $LOG
    printf "$X BRU buffer test 3 complete.\n"
    printf " " >> $LOG
    printf "\n"
    printf -- "--------------------------------\n" >> $LOG
  done
done

printf "All speed tests complete!\n"
printf "All speed tests complete!\n" >> $LOG

$REWIND

exit 0

### EOF ###